import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import javax.swing.JOptionPane; 
import controlP5.*; 
import java.text.DecimalFormat; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class AirScopeDemo extends PApplet {

//Toy with these
final int MAXPARTICLES = 2000;
final int MAXSPEED = 65;
final int MAXTEMP = 150;
final int WINDOWX = 600;




//import processing.serial.*;

ControlP5 cp5;
//Serial myPort;
//boolean connected = false;

PImage oxygen, nitrogen, co2, water, argon;

int NUMPARTICLES = MAXPARTICLES;

Particle[] particles = new Particle[NUMPARTICLES];

float water_prop;
float oxygen_prop; 
float nitrogen_prop; 
float co2_prop;
float argon_prop;

PVector wind = new PVector(0,5);
float direction_radians;
int direction_degrees;
float wind_mph;
float wind_kph;
float wind_val;
String wind_status;

float temp_val;
float temp_c;
float temp_f; //With warming
float temp_raw_f; //Without warming
String temp_status;

float windchill_f;
float windchill_c;
float heatindex_f;
float heatindex_c;

float rel_hum;
float dew_point_f;
float dew_point_c;
float pressure;
float pressure_in;
String dew_status;
String fog_status;

float warming_f;
float warming_c;
float effect_f;

boolean metric = false;
String status;

//Sizes the window, sets parameters to defaults, and creates particles
public void setup() {
  
  
  //connect();
  
  setupGUI();
  
  oxygen = loadImage("oxygen.png");
  nitrogen = loadImage("nitrogen.png");
  co2 = loadImage("co2.png");
  argon = loadImage("argon.png");
  water = loadImage("water.png");
  
  status = "AirScope is using default values.";  
  
  setWind(90, 5);
  normalAir();
  setTemp(50);
  setHum(.55f, 1013.25f);
  
  JOptionPane.showMessageDialog(frame, "AirScope is the intellectual property of Marc Campasano, protected by a Creative Commons license.\nYou may use and share AirScope as you like, but may not alter it or use it for commercial purposes.\nYou must give credit to Marc Campasano.", "Creative Commons License", JOptionPane.INFORMATION_MESSAGE);
    
  reboot();
}

//Clears the screen and updates
public void draw() {
  background(255);
  
  for (int i = 0; i < NUMPARTICLES; i++){
     particles[i].update();
  }
  
  fill(255);
  rect(WINDOWX, 0, width-WINDOWX, height);
  line(WINDOWX, 0, WINDOWX, height);
  
  GUIdraw();
   
}


//Resets all the particles. Use when changing the molecular composition.
public void reboot() {
    
  for (int i = 0; i < MAXPARTICLES; i++) {
      particles[i] = new Particle();
  }
}

//Sets air composition to Earth's norm
public void normalAir() {
  setAir(.0004f, .2095f, .7809f, .00933f);
}


//This is the big function that handles weather
public void setWind(int wind_dir, float wind_sp) { 
   
  //wind_dir is incoming wind direction in degrees
  direction_degrees = wind_dir;
  
  while (direction_degrees >= 360) {
    direction_degrees -= 360;
  }
  
  while (direction_degrees < 0) {
    direction_degrees += 360;
  }
    
  direction_radians = radians(wind_dir) - HALF_PI; //This translates to the coordinate system
  
  //wind_sp is incoming wind mph
  
  wind_mph = wind_sp;
  
  if (wind_mph < 0 || temp_f <= -459.67f) { //No wind at absolute 0
    wind_mph = 0;
  }
  
  wind_kph = wind_mph * 1.60934f;
  wind_val = map(wind_mph, 0, 200, 0, MAXSPEED);
  
  if (wind_val > MAXSPEED) {
    wind_val = MAXSPEED;
  }
  
  //From Wikipedia on the Beaufort scale and the Saffic-Simpson hurricane scale
  if (wind_mph < 1) {
    wind_status = "The air is still.";
  } else if (wind_mph < 4) {
    wind_status = "There's a tiny breeze.";
  } else if (wind_mph < 8) {
    wind_status = "You can feel a light breeze.";
  } else if (wind_mph < 13) {
    wind_status = "The wind can blow leaves around.";
  } else if (wind_mph < 19) {
    wind_status = "The wind can move small tree branches.";
  } else if (wind_mph < 25) {
    wind_status = "The wind get move larger tree branches.";
  } else if (wind_mph < 32) {
    wind_status = "The wind is getting strong.";
  } else if (wind_mph < 39) {
    wind_status = "It's hard to walk against the wind.";
  } else if (wind_mph < 47) {
    wind_status = "Take cover! Gale force winds!";
  } else if (wind_mph < 55) {
    wind_status = "The wind can blow branches off trees!";
  } else if (wind_mph < 64) {
    wind_status = "The wind can knock down trees!";
  } else if (wind_mph < 73) {
    wind_status = "The wind can seriously damage buildings!";
  } else if (wind_mph < 96) {
    wind_status = "A category 1 hurricane! We might lose power!";
  } else if (wind_mph < 111) {
    wind_status = "Category 2 hurricane! A complete power outage!";
  } else if (wind_mph < 130) {
    wind_status = "Category 3 hurricane!";
  } else if (wind_mph < 157) {
    wind_status = "Category 4 hurricane! The area will be devastated for weeks!";
  } else if (wind_mph < 252) {
    wind_status = "Category 5 hurricane! This would destroy most houses!";
  } else {
    wind_status = "Faster winds than Earth's world record!";
  }
  
  
  wind = PVector.fromAngle(direction_radians, wind);
  wind.setMag(wind_val);
  updateWindChill();
}


public void setTemp(float temp) {
  
  //temp is incoming temperature in Fahrenheit
  temp_raw_f = temp;
  
  updateWarming();
  
  if (temp_f <= -459.67f) {
    temp_f = -459.67f;
    setWind(direction_degrees, 0);
    temp_val = 0;
  } else {
    temp_val = map(temp_f, -100, 2000, 0.1f, MAXTEMP);
    
    if (temp_val > MAXTEMP) {
      temp_val = MAXTEMP;
    } else if (temp_val < 0.1f) {
      temp_val = 0.1f;
    }
    
  }
  
  temp_c = FtoC(temp_f);
  
  
  updateWater();
  updateWindChill();
  updateHeatIndex();
  
  if (temp_f > 9941) {
    temp_status = "Are you on the Sun? Because that's how hot it is.";
  } else if (temp_f > 212) {
    temp_status = "Water boils at temperatures this high!";  
  } else if (temp_f > 129) {
    temp_status = "It's hotter than Earth's world record!";
  } else if (temp_f > 105) {
    temp_status = "It's extremely hot! Be careful!";
  } else if (temp_f > 90) {
    temp_status = "It's very hot! Stay in the shade!";
  } else if (temp_f > 77) {
    temp_status = "It's hot! Drink lots of water!";
  } else if (temp_f > 66) {
    temp_status = "It's warm.";
  } else if (temp_f > 55) {
    temp_status = "It's a mild temperature.";
  } else if (temp_f > 46) {
    temp_status = "The air is cool.";
  } else if (temp_f > 33) {
    temp_status = "It's a bit cold.";
  } else if (temp_f > 23) {
    temp_status = "It's cold enough to freeze water.";
  } else if (temp_f > 0) {
    temp_status = "It's pretty cold!";
  } else if (temp_f > -40) {
    temp_status = "It's extremely cold!";
  }  else if (temp_f > -128.6f) {
    temp_status = "It's super-duper cold!";
  } else if (temp_f > -459.67f) {
    temp_status = "It's colder than Earth's world record!";
  } else {
    temp_status = "You've reached absolute zero. It can't get any colder.";
  }
    
    
}

public void setHum(float hum, float bar_pres)  {
  //hum and bar_pres are incoming relative humidity and barometric pressure in millibars (hPa)
  rel_hum = hum;
  pressure = bar_pres;
  
  if (rel_hum < 0) { //Min and max relative humidity
    rel_hum = 0;
  } else if (rel_hum > 1) {
    rel_hum = 1;
  }
  
  if (pressure < 0) { //Min pressure
    pressure = 0;
  }
  
  NUMPARTICLES = PApplet.parseInt(map(pressure, 0, 2000, 0, 1500));
  
  if (NUMPARTICLES > MAXPARTICLES) {
    NUMPARTICLES = MAXPARTICLES;
  }
  
  pressure_in = pressure * .0295301f;
  
  updateWater();
  updateHeatIndex();
}

//This is the big function that handles chemistry
public void setAir(float co2p, float oxyp, float nitp, float argonp){
  
  if (co2p < .000001f) {
    co2p = .000001f;
  } else if (co2p > 1) {
    co2p = 1;
  }
  
  if (co2p >= 0 && oxyp >= 0 && nitp >= 0 && argonp >= 0) {
    co2_prop = co2p;
    oxygen_prop = oxyp;
    nitrogen_prop = nitp;
    argon_prop = argonp;
    updateWarming();
  }
  
}

public void updateWater() {
  //RelHum = PW/PWS, water proportion = PW / Ptot-PW, these equations comes from
  // http://www.vaisala.com/Vaisala%20Documents/Application%20notes/Humidity_Conversion_Formulas_B210973EN-F.pdf, equation 6
  
  float A = 6.116441f;
  float m = 7.591386f;
  float Tn = 240.7263f;
  
  float PWS = A * pow(10, (m*temp_c)/(temp_c+Tn));
  float PW = PWS*rel_hum;
  water_prop = PW / pressure;
  
  if (water_prop > .06f) { //Hard limit on water vapor or else things break at the extremes
    water_prop = .06f;
  }
  
  float logTen = log(PW/A) / log(10);
  dew_point_c = Tn / (m/logTen - 1);
  
  if (Float.isNaN(dew_point_c)){
    dew_point_c = temp_c;
  }
  
  dew_point_f = CtoF(dew_point_c);
  
  if (dew_point_f == temp_f && temp_f > 32) {
    fog_status = "It's foggy and there's dew on some surfaces.";
  } else if (dew_point_f == temp_f) {
    fog_status = "It's foggy and there's frost on some surfaces.";
  } else if (temp_f - dew_point_f < 4) {
    fog_status = "It's foggy.";
  } else {
    fog_status = "";
  }
  
  //Wikipedia
  if (dew_point_f > 80) {
    dew_status = "It is dangerously humid.";
  } else if (dew_point_f > 74) {
    dew_status = "It feels extremely humid.";
  } else if (dew_point_f > 69) {
    dew_status = "It feels uncomfortably humid.";
  } else if (dew_point_f > 64) {
    dew_status = "It feels pretty humid.";
  } else if (dew_point_f > 59) {
    dew_status = "It feels a little humid.";
  } else {
    dew_status ="";
  }
}

public void updateWindChill(){
  
  //From wikipedia
  
  if (temp_f <= 50 && wind_mph > 3) {
    windchill_f = 35.74f + 0.6215f*temp_f - 35.75f*pow(wind_mph,0.16f) + 0.4275f*temp_f*pow(wind_mph,0.16f);  
  } else {
    windchill_f = temp_f;
  }
  
  windchill_c = FtoC(windchill_f);
}

public void updateHeatIndex(){
  //From wikipedia
  
  if (temp_f >= 70 && temp_f <= 115 && rel_hum <= 80){
    
    float c1 = 0.363445176f;
    float c2 = 0.988622465f;
    float c3 = 4.777114035f;
    float c4 = -0.114037667f;
    float c5 = -0.000850208f;
    float c6 = -0.020716198f;
    float c7 = 0.000687678f;
    float c8 = 0.000274954f;
    float R = rel_hum*100;
    
    heatindex_f = c1 + c2*temp_f + c3*R + c4*temp_f*R + c5*temp_f*temp_f + c6*R*R + c7*temp_f*temp_f*R + c8*temp_f*R*R;
  } else {
    heatindex_f = temp_f;
  }
  
  heatindex_c = FtoC(heatindex_f);
  
  
}

public void updateWarming(){
  
  //Effect of global warming over pre-industrial
  warming_c = 0.8f * 5.35f * log(co2_prop/.000280f); //From Wikipedia, "Radiative Forcing"  
  warming_f = warming_c * 1.8f;
  
  //Effect of global warming over default values
  float effect_c = 0.8f * 5.35f * log(co2_prop/.0004f);
  float effect_f = effect_c * 1.8f;
  temp_f = temp_raw_f + effect_f;
  temp_c = FtoC(temp_f);
  
}

public float FtoC(float F){
  return (F-32)/1.8f;
}

public float CtoF(float C) {
  return C*1.8f + 32;
}

//void serialEvent(Serial myPort) {
//  String data = myPort.readString();
//  int begin = data.indexOf('B');
  
//    data = data.substring(begin);
  
//    String[] report = split(data, '-');
//    float getHum = float(report[1]) / 100;
//    float getTemp = float(report[2]);
//    float getPres = float(report[3]) / 100;
  
//    normalAir();
//    setWind(0, 0);
//    setTemp(getTemp);
//    setHum(getHum, getPres);
//    status = "AirScope is using data (not wind) from the lens.";
//    reboot();

//}

//void connect (){
//  if (Serial.list().length > 0) {
//    String portName = Serial.list()[0];
//    delay(1000);
//    myPort = new Serial(this, portName, 9600);
//    myPort.bufferUntil('A');
//    connected = true;
//  }
//}
public void speed_plus(int theValue){
  setWind(direction_degrees, wind_mph+5);
  tinkerStatus();
}

public void speed_minus(int theValue){
  setWind(direction_degrees, wind_mph-5);
  tinkerStatus();
}

public void speed_set(int theValue){
  
  float speed;
  
  if (metric) {
    speed = PApplet.parseFloat(JOptionPane.showInputDialog(frame, "Enter windspeed (kph):", "Set Windspeed", JOptionPane.PLAIN_MESSAGE));
    speed = speed * 0.621371f;
  } else {
    speed = PApplet.parseFloat(JOptionPane.showInputDialog(frame, "Enter windspeed (mph):", "Set Windspeed", JOptionPane.PLAIN_MESSAGE));
  }
  
  if (!Float.isNaN(speed) && speed >= 0){
    setWind(direction_degrees, speed);
    tinkerStatus();
  } else {
    errorStatus();
  }
}

public void direction_plus(int theValue){
  setWind(direction_degrees + 15, wind_mph);
  tinkerStatus();
}

public void direction_minus(int theValue){
  setWind(direction_degrees - 15, wind_mph);
  tinkerStatus();
}

public void direction_set(int theValue){
  int dir = PApplet.parseInt(JOptionPane.showInputDialog(frame, "Enter direction (degrees from north)", "Set Wind Direction", JOptionPane.PLAIN_MESSAGE));
  
  if (!Float.isNaN(dir)){
    setWind(dir, wind_mph);
    tinkerStatus();
  } else {
    errorStatus();
  }
}

public void temp_plus(int theValue){
  setTemp(temp_f+5);
  tinkerStatus();
}

public void temp_minus(int theValue){
  setTemp(temp_f-5);
  tinkerStatus();
}

public void temp_set(int theValue){
  
  float temp;
  
  if (metric) {
    temp = PApplet.parseFloat(JOptionPane.showInputDialog(frame, "Enter temperature (C):", "Set Temperature", JOptionPane.PLAIN_MESSAGE));
    temp = CtoF(temp);
  } else {
    temp = PApplet.parseFloat(JOptionPane.showInputDialog(frame, "Enter temperature (F)", "Set Temperature", JOptionPane.PLAIN_MESSAGE));
  }
  
  if (!Float.isNaN(temp)){
    normalAir();
    setTemp(temp);
    status = "Temperature set. Air composition reset to normal Earth.";
  } else {
    errorStatus();
  }
}

public void hum_plus(int theValue){
  setHum(rel_hum+.05f, pressure);
  tinkerStatus();
}

public void hum_minus(int theValue){
  setHum(rel_hum-.05f, pressure);
  tinkerStatus();
}

public void hum_set(int theValue){
  float hum = PApplet.parseFloat(JOptionPane.showInputDialog(frame, "Enter relative humidity (%):", "Set Humidity", JOptionPane.PLAIN_MESSAGE));
  
  if (!Float.isNaN(hum) && hum >= 0 && hum <= 100){
    setHum(hum/100, pressure);
    tinkerStatus();
  } else {
    errorStatus();
  }
}

public void pres_plus(int theValue){
  setHum(rel_hum, pressure+1);
  tinkerStatus();
}

public void pres_minus(int theValue){
  setHum(rel_hum, pressure-1);
  tinkerStatus();
}

public void pres_set(int theValue){
  
  float pres;
  
  if (metric) {
    pres = PApplet.parseFloat(JOptionPane.showInputDialog(frame, "Enter barometric pressure (millibars):", "Set Pressure", JOptionPane.PLAIN_MESSAGE));
  } else {
    pres = PApplet.parseFloat(JOptionPane.showInputDialog(frame, "Enter barometric pressure (inches):", "Set Pressure", JOptionPane.PLAIN_MESSAGE));
    pres = pres * 33.8639f;
  }
  
  if (!Float.isNaN(pres) && pres >= 0){
    setHum(rel_hum, pres);
    reboot();
    rebootStatus();
  } else {
    errorStatus();
  }
}
  
public void co2_plus(int theValue){
  setAir(co2_prop + .000025f, .2095f, .7809f, .00933f);
  status = "CO2 added. Air composition reset to normal Earth.";
}

public void co2_minus(int theValue){
  setAir(co2_prop - .000025f, .2095f, .7809f, .00933f);
  status = "CO2 removed. Air composition reset to normal Earth.";
}

public void co2_set(int theValue){
  float co2 = PApplet.parseFloat(JOptionPane.showInputDialog(frame, "Enter carbon dioxide (ppm):", "Set Carbon Dioxide", JOptionPane.PLAIN_MESSAGE));
  co2 = co2/1000000;
  
  if (!Float.isNaN(co2) && co2 >= 0){
     setAir(co2, oxygen_prop, nitrogen_prop, argon_prop);
     reboot();
     rebootStatus();
  } else {
    errorStatus();
  }
    
}

public void system_switch(boolean theValue){
  
  metric = !metric;
  
  if (metric) {
    cp5.getController("system_switch").setCaptionLabel("Metric");
  } else {
    cp5.getController("system_switch").setCaptionLabel("U.S.");
  }
}

//CALL ON THE WEATHER GODS -- Uses the OpenWeather API  
public void internet(int theValue) {
  
  String ZIP = JOptionPane.showInputDialog(frame, "Enter a ZIP code:", "Get Weather Data", JOptionPane.PLAIN_MESSAGE);
  String URL = "http://api.openweathermap.org/data/2.5/weather?zip=" + ZIP + "&units=imperial&forecast?id=524901&APPID=f76d1e93101b404983a4d64099be7379";
                  
  JSONObject weatherConditions= loadJSONObject(URL);
  JSONObject wind;
          
  //Error checking for invalid input
   if (weatherConditions.isNull("main")) {
            errorStatus();
   } else {
          
          wind = weatherConditions.getJSONObject("wind");
          weatherConditions = weatherConditions.getJSONObject("main");
          float get_temp = weatherConditions.getFloat("temp");
          //JOptionPane.showMessageDialog(frame, "The temp is " + get_temp, "Test", JOptionPane.INFORMATION_MESSAGE);
          float get_hum = weatherConditions.getFloat("humidity");
          //JOptionPane.showMessageDialog(frame, "The humidity is " + get_hum, "Test", JOptionPane.INFORMATION_MESSAGE);
          float get_pres = weatherConditions.getFloat("pressure");
          //JOptionPane.showMessageDialog(frame, "The pressure is " + get_pres, "Test", JOptionPane.INFORMATION_MESSAGE);
          float get_windspeed = wind.getFloat("speed");
          //JOptionPane.showMessageDialog(frame, "The windspeed is " + get_windspeed, "Test", JOptionPane.INFORMATION_MESSAGE);
          int get_direction = wind.getInt("deg");
          //JOptionPane.showMessageDialog(frame, "The wind direction is " + get_direction, "Test", JOptionPane.INFORMATION_MESSAGE);
          
          status = "AirScope is using real weather data.";
          
          normalAir();
          setWind(get_direction, get_windspeed);
          setTemp(get_temp);
          setHum(get_hum, get_pres);
          
          reboot();
    }
}

public void venus(int theValue){
  setAir(.965f,0,.035f,0);
  setWind(PApplet.parseInt(random(360)), 220);
  setTemp(872);
  setHum(0, 93);
  status= "AirScope is showing the air on planet Venus.";
  reboot();
}

public void rainforest(int theValue){
  normalAir();
  setWind(PApplet.parseInt(random(360)), 0);
  setTemp(80);
  setHum(.83f, 1015);
  status= "AirScope is showing air from the Amazon rainforest.";      
  reboot();          
}

public void sahara(int theValue){
  normalAir();
  setWind(PApplet.parseInt(random(360)), 0);
  setTemp(104);
  setHum(0.05f, 1030);
  status= "AirScope is showing air from the summer Sahara Desert.";
  reboot();         
}

public void southpole(int theValue){
  normalAir(); //https://icecube.wisc.edu/pole/weather
  setWind(20, 12.3f);
  setTemp(-57.1f);
  setHum(0, 681.2f);
  status= "AirScope is showing air from the South Pole.";
  reboot();        
}

public void probe(int theValue){
  
 JOptionPane.showMessageDialog(frame, "Not included in this demo. This button would grab temperature, humidity, and pressure data from the air if you had the special AirScope magnifying lens.", "Lens Function", JOptionPane.PLAIN_MESSAGE);
 
}

public void about(int theValue) {
  JOptionPane.showMessageDialog(frame, "Designed and programmed by Marc Campasano\nfor 'Beyond Bits and Atoms' at Stanford University\n\nLocal weather from OpenWeather", "About AirScope", JOptionPane.PLAIN_MESSAGE);
}

public void tinkerStatus() {
  status = "You are experimenting with the air values.";
}

public void errorStatus() {
  status = "That didn't quite work. Try again.";
}

public void rebootStatus() {
  status = "AirScope regenerated its visualization.";
}
public void setupGUI() {
 cp5 = new ControlP5(this);
  
  cp5.addButton("speed_plus")
  .setPosition(WINDOWX+10, 40)
  .setSize(15, 15)
  .setCaptionLabel("+");
  
  cp5.addButton("speed_minus")
  .setPosition(WINDOWX+30, 40)
  .setSize(15, 15)
  .setCaptionLabel("-");
  
  cp5.addButton("speed_set")
  .setPosition(WINDOWX+50, 40)
  .setSize(30, 15)
  .setCaptionLabel("set");
  
  cp5.addButton("direction_plus")
  .setPosition(WINDOWX+10, 60)
  .setSize(15, 15)
  .setCaptionLabel("+");
  
  cp5.addButton("direction_minus")
  .setPosition(WINDOWX+30, 60)
  .setSize(15, 15)
  .setCaptionLabel("-");
  
  cp5.addButton("direction_set")
  .setPosition(WINDOWX+50, 60)
  .setSize(30, 15)
  .setCaptionLabel("set");
  
  cp5.addButton("temp_plus")
  .setPosition(WINDOWX+10, 115)
  .setSize(15, 15)
  .setCaptionLabel("+");
  
  cp5.addButton("temp_minus")
  .setPosition(WINDOWX+30, 115)
  .setSize(15, 15)
  .setCaptionLabel("-");
  
  cp5.addButton("temp_set")
  .setPosition(WINDOWX+50, 115)
  .setSize(30, 15)
  .setCaptionLabel("set");
  
  cp5.addButton("hum_plus")
  .setPosition(WINDOWX+10, 210)
  .setSize(15, 15)
  .setCaptionLabel("+");
  
  cp5.addButton("hum_minus")
  .setPosition(WINDOWX+30, 210)
  .setSize(15, 15)
  .setCaptionLabel("-");
  
  cp5.addButton("hum_set")
  .setPosition(WINDOWX+50, 210)
  .setSize(30, 15)
  .setCaptionLabel("set");
  
  cp5.addButton("pres_plus")
  .setPosition(WINDOWX+10, 230)
  .setSize(15, 15)
  .setCaptionLabel("+");
  
  cp5.addButton("pres_minus")
  .setPosition(WINDOWX+30, 230)
  .setSize(15, 15)
  .setCaptionLabel("-");
  
  cp5.addButton("pres_set")
  .setPosition(WINDOWX+50, 230)
  .setSize(30, 15)
  .setCaptionLabel("set");
  
  cp5.addButton("co2_plus")
  .setPosition(WINDOWX+10, 305)
  .setSize(15, 15)
  .setCaptionLabel("+");
  
  cp5.addButton("co2_minus")
  .setPosition(WINDOWX+30, 305)
  .setSize(15, 15)
  .setCaptionLabel("-");
  
  cp5.addButton("co2_set")
  .setPosition(WINDOWX+50, 305)
  .setSize(30, 15)
  .setCaptionLabel("set");
  
  cp5.addToggle("system_switch")
  .setPosition(width-210, height-50)
  .setSize(100, 30)
  .setMode(ControlP5.SWITCH)
  .setColorCaptionLabel(0)
  .setCaptionLabel("U.S.");
  
  cp5.addButton("probe")
  .setPosition(650, height - 195)
  .setSize(100, 65)
  .setCaptionLabel("Use Lens");
  
  cp5.addButton("internet")
  .setPosition(755, height - 195)
  .setSize(100, 65)
  .setCaptionLabel("Enter ZIP");
  
  cp5.addButton("venus")
  .setPosition(1005, height-160)
  .setSize(100, 30)
  .setCaptionLabel("Venus");
  
  cp5.addButton("southpole")
  .setPosition(900, height-160)
  .setSize(100, 30)
  .setCaptionLabel("South Pole");
  
  cp5.addButton("sahara")
  .setPosition(900, height-195)
  .setSize(100, 30)
  .setCaptionLabel("Sahara Desert");
  
  cp5.addButton("rainforest")
  .setPosition(1005, height-195)
  .setSize(100, 30)
  .setCaptionLabel("Amazon");
  
  cp5.addButton("about")
  .setPosition(width - 105, height-50)
  .setSize(100, 30)
  .setCaptionLabel("About");
}

public void GUIdraw() {
  
  DecimalFormat df = new DecimalFormat("0.00");
  
  float display_wind = wind_mph;
  float display_temp = temp_f;
  float display_windchill = windchill_f;
  float display_heatindex = heatindex_f;
  float display_dewpoint = dew_point_f;
  float display_warming = warming_f;
  float display_pressure = pressure_in;
  String distance_unit = " mph";
  String temp_unit = "\u00b0F";
  String units = "U.S";
  String pressure_unit = " inches";
  
  if (metric) {
    display_wind = wind_kph;
    display_temp = temp_c;
    display_windchill = windchill_c;
    display_heatindex = heatindex_c;
    display_dewpoint = dew_point_c;
    display_warming = warming_c;
    display_pressure = pressure;
    pressure_unit = " millibars";
    distance_unit = " kph";
    temp_unit = "\u00b0C";
  }
  
  fill(0);

  textSize(14);
  textAlign(RIGHT);
  text(wind_status, width-5, 25);
  text(dew_status, width-5, 195);
  text(temp_status, width-5, 100);
  text(fog_status, width-5, 263);
  
  textAlign(LEFT);
  text(status, WINDOWX + 5, height - 20);
  
  
  textSize(24);
  text("Real Air", 650, height - 202);
  text("Explore Biomes", 900, height - 202);
  text("Wind", WINDOWX+5, 30);
  textSize(14);
  text("Speed: " + df.format(display_wind) + distance_unit, WINDOWX+90, 53);
  text("Direction: " + direction_degrees + "\u00b0 from north", WINDOWX+90, 73);
  textSize(24);
  text("Temperature", WINDOWX+5, 105);
  textSize(14);
  text("Temperature: " + df.format(display_temp) + temp_unit, WINDOWX+90, 128);
  
  if (windchill_f != temp_f){
    text("Wind Chill: " + df.format(display_windchill) + temp_unit, WINDOWX+90, 148);
  } else if (heatindex_f != temp_f) {  
    text("Heat Index: " + df.format(display_heatindex) + temp_unit, WINDOWX+90, 148);
  }
    
  textSize(24);
  text("Humidity and Pressure", WINDOWX+5, 200);
  textSize(14);
  text("Relative Humidity: " + df.format(rel_hum*100) + "%", WINDOWX+90, 223);
  text("Barometric Pressure: "+ df.format(display_pressure) + pressure_unit, WINDOWX+90, 243);
  text("Dew Point: " + df.format(display_dewpoint) + temp_unit, WINDOWX+90, 263);
  textSize(24);
  text("Carbon Dioxide", WINDOWX+5, 295);
  textSize(14);
  text("Parts per Million: " + ceil(co2_prop * 1000000), WINDOWX+90, 318);
  text("Greenhouse Effect: " + df.format(display_warming) + temp_unit, WINDOWX+90, 338);
  
    
}
//Particles are a coordinate pair and an image
class Particle {
  float xpos, ypos;
  PImage img;
  
  Particle() {
    xpos = random(-50, WINDOWX+50);
    ypos = random(-50, height+50);
    img = getImage();
  }
 
 //Uses the air composition to determine each particle's image
 public PImage getImage() {
   float rand = random(1);
   
   if (rand < water_prop) {
     return water;
   }
      
   rand = random(1);
   
   if (rand < co2_prop) {
     return co2;
   } else if (rand < co2_prop+oxygen_prop) {
     return oxygen;
   } else if (rand < co2_prop+oxygen_prop+argon_prop) {
     return argon;
   } else {
     return nitrogen;
   } 
   
 }
  
 //Consults the windspeed to update movement
  public void update() {
    
    
    
    //Moves with the wind, vibrates with temperature
    xpos = xpos + wind.x + random(-temp_val, temp_val);
    ypos = ypos + wind.y + random(-temp_val, temp_val);
    
    //Refreshes at other side of screen when out of bounds
    if (xpos > WINDOWX + 60) {
      xpos = -50;
      ypos = random(height);
      img = getImage();
    } else if (xpos < -60) {
      xpos = WINDOWX+50;
      ypos = random(height);
      img = getImage();
    }
    
    if (ypos > height + 60) {
      ypos = -50;
      xpos = random(WINDOWX);
      img = getImage();
    } else if (ypos < -60) {
      ypos = height+50;
      xpos = random(WINDOWX);
      img = getImage();
    }
    
    //Draw the particle
    image(img, xpos, ypos);
  }
}
  public void settings() {  size(1200, 600); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "AirScopeDemo" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
