//Toy with these
final int MAXPARTICLES = 2000;
final int MAXSPEED = 65;
final int MAXTEMP = 150;
final int WINDOWX = 600;

import javax.swing.JOptionPane;
import controlP5.*;
import java.text.DecimalFormat;
//import processing.serial.*;

ControlP5 cp5;
//Serial myPort;
//boolean connected = false;

PImage oxygen, nitrogen, co2, water, argon;

int NUMPARTICLES = MAXPARTICLES;

Particle[] particles = new Particle[NUMPARTICLES];

float water_prop;
float oxygen_prop; 
float nitrogen_prop; 
float co2_prop;
float argon_prop;

PVector wind = new PVector(0,5);
float direction_radians;
int direction_degrees;
float wind_mph;
float wind_kph;
float wind_val;
String wind_status;

float temp_val;
float temp_c;
float temp_f; //With warming
float temp_raw_f; //Without warming
String temp_status;

float windchill_f;
float windchill_c;
float heatindex_f;
float heatindex_c;

float rel_hum;
float dew_point_f;
float dew_point_c;
float pressure;
float pressure_in;
String dew_status;
String fog_status;

float warming_f;
float warming_c;
float effect_f;

boolean metric = false;
String status;

//Sizes the window, sets parameters to defaults, and creates particles
void setup() {
  size(1200, 600);
  
  //connect();
  
  setupGUI();
  
  oxygen = loadImage("oxygen.png");
  nitrogen = loadImage("nitrogen.png");
  co2 = loadImage("co2.png");
  argon = loadImage("argon.png");
  water = loadImage("water.png");
  
  status = "AirScope is using default values.";  
  
  setWind(90, 5);
  normalAir();
  setTemp(50);
  setHum(.55, 1013.25);
  
  JOptionPane.showMessageDialog(frame, "AirScope is the intellectual property of Marc Campasano, protected by a Creative Commons license.\nYou may use and share AirScope as you like, but may not alter it or use it for commercial purposes.\nYou must give credit to Marc Campasano.", "Creative Commons License", JOptionPane.INFORMATION_MESSAGE);
    
  reboot();
}

//Clears the screen and updates
void draw() {
  background(255);
  
  for (int i = 0; i < NUMPARTICLES; i++){
     particles[i].update();
  }
  
  fill(255);
  rect(WINDOWX, 0, width-WINDOWX, height);
  line(WINDOWX, 0, WINDOWX, height);
  
  GUIdraw();
   
}


//Resets all the particles. Use when changing the molecular composition.
void reboot() {
    
  for (int i = 0; i < MAXPARTICLES; i++) {
      particles[i] = new Particle();
  }
}

//Sets air composition to Earth's norm
void normalAir() {
  setAir(.0004, .2095, .7809, .00933);
}


//This is the big function that handles weather
void setWind(int wind_dir, float wind_sp) { 
   
  //wind_dir is incoming wind direction in degrees
  direction_degrees = wind_dir;
  
  while (direction_degrees >= 360) {
    direction_degrees -= 360;
  }
  
  while (direction_degrees < 0) {
    direction_degrees += 360;
  }
    
  direction_radians = radians(wind_dir) - HALF_PI; //This translates to the coordinate system
  
  //wind_sp is incoming wind mph
  
  wind_mph = wind_sp;
  
  if (wind_mph < 0 || temp_f <= -459.67) { //No wind at absolute 0
    wind_mph = 0;
  }
  
  wind_kph = wind_mph * 1.60934;
  wind_val = map(wind_mph, 0, 200, 0, MAXSPEED);
  
  if (wind_val > MAXSPEED) {
    wind_val = MAXSPEED;
  }
  
  //From Wikipedia on the Beaufort scale and the Saffic-Simpson hurricane scale
  if (wind_mph < 1) {
    wind_status = "The air is still.";
  } else if (wind_mph < 4) {
    wind_status = "There's a tiny breeze.";
  } else if (wind_mph < 8) {
    wind_status = "You can feel a light breeze.";
  } else if (wind_mph < 13) {
    wind_status = "The wind can blow leaves around.";
  } else if (wind_mph < 19) {
    wind_status = "The wind can move small tree branches.";
  } else if (wind_mph < 25) {
    wind_status = "The wind get move larger tree branches.";
  } else if (wind_mph < 32) {
    wind_status = "The wind is getting strong.";
  } else if (wind_mph < 39) {
    wind_status = "It's hard to walk against the wind.";
  } else if (wind_mph < 47) {
    wind_status = "Take cover! Gale force winds!";
  } else if (wind_mph < 55) {
    wind_status = "The wind can blow branches off trees!";
  } else if (wind_mph < 64) {
    wind_status = "The wind can knock down trees!";
  } else if (wind_mph < 73) {
    wind_status = "The wind can seriously damage buildings!";
  } else if (wind_mph < 96) {
    wind_status = "A category 1 hurricane! We might lose power!";
  } else if (wind_mph < 111) {
    wind_status = "Category 2 hurricane! A complete power outage!";
  } else if (wind_mph < 130) {
    wind_status = "Category 3 hurricane!";
  } else if (wind_mph < 157) {
    wind_status = "Category 4 hurricane! The area will be devastated for weeks!";
  } else if (wind_mph < 252) {
    wind_status = "Category 5 hurricane! This would destroy most houses!";
  } else {
    wind_status = "Faster winds than Earth's world record!";
  }
  
  
  wind = PVector.fromAngle(direction_radians, wind);
  wind.setMag(wind_val);
  updateWindChill();
}


void setTemp(float temp) {
  
  //temp is incoming temperature in Fahrenheit
  temp_raw_f = temp;
  
  updateWarming();
  
  if (temp_f <= -459.67) {
    temp_f = -459.67;
    setWind(direction_degrees, 0);
    temp_val = 0;
  } else {
    temp_val = map(temp_f, -100, 2000, 0.1, MAXTEMP);
    
    if (temp_val > MAXTEMP) {
      temp_val = MAXTEMP;
    } else if (temp_val < 0.1) {
      temp_val = 0.1;
    }
    
  }
  
  temp_c = FtoC(temp_f);
  
  
  updateWater();
  updateWindChill();
  updateHeatIndex();
  
  if (temp_f > 9941) {
    temp_status = "Are you on the Sun? Because that's how hot it is.";
  } else if (temp_f > 212) {
    temp_status = "Water boils at temperatures this high!";  
  } else if (temp_f > 129) {
    temp_status = "It's hotter than Earth's world record!";
  } else if (temp_f > 105) {
    temp_status = "It's extremely hot! Be careful!";
  } else if (temp_f > 90) {
    temp_status = "It's very hot! Stay in the shade!";
  } else if (temp_f > 77) {
    temp_status = "It's hot! Drink lots of water!";
  } else if (temp_f > 66) {
    temp_status = "It's warm.";
  } else if (temp_f > 55) {
    temp_status = "It's a mild temperature.";
  } else if (temp_f > 46) {
    temp_status = "The air is cool.";
  } else if (temp_f > 33) {
    temp_status = "It's a bit cold.";
  } else if (temp_f > 23) {
    temp_status = "It's cold enough to freeze water.";
  } else if (temp_f > 0) {
    temp_status = "It's pretty cold!";
  } else if (temp_f > -40) {
    temp_status = "It's extremely cold!";
  }  else if (temp_f > -128.6) {
    temp_status = "It's super-duper cold!";
  } else if (temp_f > -459.67) {
    temp_status = "It's colder than Earth's world record!";
  } else {
    temp_status = "You've reached absolute zero. It can't get any colder.";
  }
    
    
}

void setHum(float hum, float bar_pres)  {
  //hum and bar_pres are incoming relative humidity and barometric pressure in millibars (hPa)
  rel_hum = hum;
  pressure = bar_pres;
  
  if (rel_hum < 0) { //Min and max relative humidity
    rel_hum = 0;
  } else if (rel_hum > 1) {
    rel_hum = 1;
  }
  
  if (pressure < 0) { //Min pressure
    pressure = 0;
  }
  
  NUMPARTICLES = int(map(pressure, 0, 2000, 0, 1500));
  
  if (NUMPARTICLES > MAXPARTICLES) {
    NUMPARTICLES = MAXPARTICLES;
  }
  
  pressure_in = pressure * .0295301;
  
  updateWater();
  updateHeatIndex();
}

//This is the big function that handles chemistry
void setAir(float co2p, float oxyp, float nitp, float argonp){
  
  if (co2p < .000001) {
    co2p = .000001;
  } else if (co2p > 1) {
    co2p = 1;
  }
  
  if (co2p >= 0 && oxyp >= 0 && nitp >= 0 && argonp >= 0) {
    co2_prop = co2p;
    oxygen_prop = oxyp;
    nitrogen_prop = nitp;
    argon_prop = argonp;
    updateWarming();
  }
  
}

void updateWater() {
  //RelHum = PW/PWS, water proportion = PW / Ptot-PW, these equations comes from
  // http://www.vaisala.com/Vaisala%20Documents/Application%20notes/Humidity_Conversion_Formulas_B210973EN-F.pdf, equation 6
  
  float A = 6.116441;
  float m = 7.591386;
  float Tn = 240.7263;
  
  float PWS = A * pow(10, (m*temp_c)/(temp_c+Tn));
  float PW = PWS*rel_hum;
  water_prop = PW / pressure;
  
  if (water_prop > .06) { //Hard limit on water vapor or else things break at the extremes
    water_prop = .06;
  }
  
  float logTen = log(PW/A) / log(10);
  dew_point_c = Tn / (m/logTen - 1);
  
  if (Float.isNaN(dew_point_c)){
    dew_point_c = temp_c;
  }
  
  dew_point_f = CtoF(dew_point_c);
  
  if (dew_point_f == temp_f && temp_f > 32) {
    fog_status = "It's foggy and there's dew on some surfaces.";
  } else if (dew_point_f == temp_f) {
    fog_status = "It's foggy and there's frost on some surfaces.";
  } else if (temp_f - dew_point_f < 4) {
    fog_status = "It's foggy.";
  } else {
    fog_status = "";
  }
  
  //Wikipedia
  if (dew_point_f > 80) {
    dew_status = "It is dangerously humid.";
  } else if (dew_point_f > 74) {
    dew_status = "It feels extremely humid.";
  } else if (dew_point_f > 69) {
    dew_status = "It feels uncomfortably humid.";
  } else if (dew_point_f > 64) {
    dew_status = "It feels pretty humid.";
  } else if (dew_point_f > 59) {
    dew_status = "It feels a little humid.";
  } else {
    dew_status ="";
  }
}

void updateWindChill(){
  
  //From wikipedia
  
  if (temp_f <= 50 && wind_mph > 3) {
    windchill_f = 35.74 + 0.6215*temp_f - 35.75*pow(wind_mph,0.16) + 0.4275*temp_f*pow(wind_mph,0.16);  
  } else {
    windchill_f = temp_f;
  }
  
  windchill_c = FtoC(windchill_f);
}

void updateHeatIndex(){
  //From wikipedia
  
  if (temp_f >= 70 && temp_f <= 115 && rel_hum <= 80){
    
    float c1 = 0.363445176;
    float c2 = 0.988622465;
    float c3 = 4.777114035;
    float c4 = -0.114037667;
    float c5 = -0.000850208;
    float c6 = -0.020716198;
    float c7 = 0.000687678;
    float c8 = 0.000274954;
    float R = rel_hum*100;
    
    heatindex_f = c1 + c2*temp_f + c3*R + c4*temp_f*R + c5*temp_f*temp_f + c6*R*R + c7*temp_f*temp_f*R + c8*temp_f*R*R;
  } else {
    heatindex_f = temp_f;
  }
  
  heatindex_c = FtoC(heatindex_f);
  
  
}

void updateWarming(){
  
  //Effect of global warming over pre-industrial
  warming_c = 0.8 * 5.35 * log(co2_prop/.000280); //From Wikipedia, "Radiative Forcing"  
  warming_f = warming_c * 1.8;
  
  //Effect of global warming over default values
  float effect_c = 0.8 * 5.35 * log(co2_prop/.0004);
  float effect_f = effect_c * 1.8;
  temp_f = temp_raw_f + effect_f;
  temp_c = FtoC(temp_f);
  
}

float FtoC(float F){
  return (F-32)/1.8;
}

float CtoF(float C) {
  return C*1.8 + 32;
}

//void serialEvent(Serial myPort) {
//  String data = myPort.readString();
//  int begin = data.indexOf('B');
  
//    data = data.substring(begin);
  
//    String[] report = split(data, '-');
//    float getHum = float(report[1]) / 100;
//    float getTemp = float(report[2]);
//    float getPres = float(report[3]) / 100;
  
//    normalAir();
//    setWind(0, 0);
//    setTemp(getTemp);
//    setHum(getHum, getPres);
//    status = "AirScope is using data (not wind) from the lens.";
//    reboot();

//}

//void connect (){
//  if (Serial.list().length > 0) {
//    String portName = Serial.list()[0];
//    delay(1000);
//    myPort = new Serial(this, portName, 9600);
//    myPort.bufferUntil('A');
//    connected = true;
//  }
//}